Imports System.Runtime.InteropServices
Imports System.ComponentModel
Imports Outlook = Microsoft.Office.Interop.Outlook
Imports System.Windows.Forms
Imports System.Diagnostics

'Add-in Express Add-in Module
<GuidAttribute("460CC007-7FFA-4B3D-BA2E-C1986C657CD8"), ProgIdAttribute("OutlookItemEvents.AddinModule"), RunInstaller(True)> _
Public Class AddinModule
    Inherits AddinExpress.MSO.ADXAddinModule
    Private WithEvents adxOutlookEvents As AddinExpress.MSO.ADXOutlookAppEvents

#Region " Component Designer generated code. "
    'Required by designer
    Private components As System.ComponentModel.IContainer

    'Required by designer - do not modify
    'the following method
    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.adxOutlookEvents = New AddinExpress.MSO.ADXOutlookAppEvents(Me.components)
        '
        'adxOutlookEvents
        '
        '
        'AddinModule
        '
        Me.AddinName = "Outlook Item Events Sample"
        Me.PageTitle = "My Custom Page"
        Me.PageType = "OutlookItemEvents.PropertyPage1"
        Me.SupportedApps = AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook

    End Sub

#End Region

#Region " ADX automatic code "

    'Required by Add-in Express - do not modify
    'the methods within this region

    Public Overrides Function GetContainer() As System.ComponentModel.IContainer
        If components Is Nothing Then
            components = New System.ComponentModel.Container
        End If
        GetContainer = components
    End Function

    <ComRegisterFunctionAttribute()> _
    Public Shared Sub AddinRegister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXRegister(t)
    End Sub

    <ComUnregisterFunctionAttribute()> _
    Public Shared Sub AddinUnregister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXUnregister(t)
    End Sub

    Public Overrides Sub UninstallControls()
        MyBase.UninstallControls()
    End Sub

#End Region

    Public Sub New()
        MyBase.New()

        Application.EnableVisualStyles()
        'This call is required by the Component Designer
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    Public ReadOnly Property OutlookApp() As Outlook._Application
        Get
            Return HostApplication
        End Get
    End Property

    Dim itemEvents As OutlookItemEventsClass1

    Private Sub AddinModule_AddinStartupComplete(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.AddinStartupComplete
        Debug.Print("The AddinStartupComplete event has occured. The add-in has been loaded by Outlook " & Me.HostMajorVersion.ToString())

        ' This creates an item of the class that handles the events of an Outlook item
        itemEvents = New OutlookItemEventsClass1(Me)
    End Sub

    Private Sub AddinModule_AddinBeginShutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.AddinBeginShutdown
        ' in Outlook 2010, this event may not occur, see the blog 'Outlook 2010 Fast Shutdown feature'
        ' at http://www.add-in-express.com/creating-addins-blog/2010/05/04/outlook2010-fast-shutdown/
        Debug.Print("The AddinBeginShutdown event has occured.")
        If itemEvents.IsConnected Then
            Debug.Print("Disconnecting from the previously connected item.")
            itemEvents.RemoveConnection()
        End If
        itemEvents = Nothing
    End Sub

    Private Sub adxOutlookEvents_InspectorActivate(ByVal sender As System.Object, ByVal inspector As System.Object, ByVal folderName As String) Handles adxOutlookEvents.InspectorActivate
        Dim itemObj As Object = (TryCast(inspector, Outlook._Inspector)).CurrentItem
        ' Now itemObj contains a COM object holding the Outlook item shown in the active inspector.
        ' Creating a COM object implies releasing it as soon as you don't need it! See how we deal with this.

        ' use late binding to retrieve the Subject property of the item
        Dim subject As String = "'" & itemObj.GetType().InvokeMember("Subject", System.Reflection.BindingFlags.GetProperty, Nothing, itemObj, Nothing).ToString() & "'"
        Marshal.ReleaseComObject(itemObj)

        Debug.Print("The InspectorActivate event has occured. The subject is " & subject & ".")

        If itemEvents.IsConnected Then
            Debug.Print("Disconnecting from the previously connected item.")
            itemEvents.RemoveConnection()
        End If

        If GetExplorersCount() > 0 Then ' don't connect if there are NO open explorers
            ' We handle all Outlook item types: MailItem, TaskItem, etc.
            ' 'True' means the events class will release the COM object holding a reference to the item shown 
            ' in the active inspector when itemEvents.RemoveConnection is called (see the code line above).
            ' Note that calling CurrentItem several times creates several COM objects, each of them must be released.
            itemEvents.ConnectTo((TryCast(inspector, Outlook._Inspector)).CurrentItem, True)
            Debug.Print("Connected to this Outlook item.")
        Else
            Debug.Print("Do not connect to this Outlook item.")
        End If
    End Sub

    Private Function GetExplorersCount() As Integer
        Dim count As Integer = 0
        Dim expls As Outlook._Explorers = Nothing

        Try
            expls = OutlookApp.Explorers
            count = expls.Count
        Catch
        Finally
            If expls IsNot Nothing Then
                Marshal.ReleaseComObject(expls)
            End If
        End Try
        Return count
    End Function


    Private Sub adxOutlookEvents_InspectorClose(ByVal sender As System.Object, ByVal inspector As System.Object, ByVal folderName As System.String) Handles adxOutlookEvents.InspectorClose
        Debug.Print("The InspectorClose event has occured.")

        If itemEvents.IsConnected Then
            Debug.Print("Disconnecting from the previously connected item.")
            itemEvents.RemoveConnection()
        End If
    End Sub
End Class

